#define OLC_SET		0
#define OLC_SHOW	1
#define OLC_REPEAT	2
#define OLC_ROOM	3
#define OLC_MOB		4
#define OLC_OBJ		5

#define OLC_COPY	0
#define OLC_NAME	1
#define OLC_DESC	2
#define OLC_ALIASES	3


#define MAX_ROOM_NAME	75
#define MAX_MOB_NAME	50
#define MAX_OBJ_NAME	50
#define MAX_ROOM_DESC	1024
#define MAX_MOB_DESC	512
#define MAX_OBJ_DESC	512
